#include "wl_def.h"

#include <stdarg.h>

#include <dingoo/ucos2.h>
#include <dingoo/entry.h>
#include <dingoo/keyboard.h>

#include <sml/graphics.h>
#include <sml/display.h>
#include <sml/control.h>
#include <sml/timer.h>

#define	JoyScaleMax		32768
#define	JoyScaleShift	8

typedef	struct {
/*	SDL_Joystick* joystick;
	
	word joyMinX;
	word joyMinY;
	word threshMinX;
	word threshMinY;
	word threshMaxX;
	word threshMaxY;
	word joyMaxX;
	word joyMaxY;
	word joyMultXL;
	word joyMultYL;
	word joyMultXH;
	word joyMultYH;*/
} JoystickDef;

static JoystickDef JoyDefs[MaxJoys];

//static SDL_Surface *surface;
//static unsigned int sdl_palettemode;

//byte *gfxbuf = NULL;
byte gfxbuf[320*240];

gfx_color a320_pal[256];
display*     gameDisplay  = NULL;

//extern void keyboard_handler(int code, int press);
//extern boolean InternalKeyboard[NumCodes];

int main(int argc, char** argv) {
  int ref = EXIT_SUCCESS;
  
  srand(OSTimeGet());

  control_init();
  gameDisplay = display_create(320, 240, 320, (DISPLAY_FORMAT_RGB565 | DISPLAY_BUFFER_STATIC), NULL, NULL);
	if(gameDisplay == NULL) {
		control_term();
		return ref;
	}
	gfx_init(gameDisplay);

  control_lock(timer_resolution / 4);
  
  gfx_render_target_clear(gfx_color_rgb(0x00, 0x00, 0x00));
  display_flip(gameDisplay);

	//ref = WolfMain(argc, argv);
  ref = WolfMain(argc, argv);
  
  gfx_term();
	display_delete(gameDisplay);
  control_term();
  
  return ref;
}

void DisplayTextSplash(byte *text);

/*
==========================
=
= Quit
=
==========================
*/

void Quit(const char *error)
{
	memptr screen = NULL;

	if (!error || !*error) {
		CA_CacheGrChunk(ORDERSCREEN);
		screen = grsegs[ORDERSCREEN];
		WriteConfig();
	} else if (error) {
		CA_CacheGrChunk(ERRORSCREEN);
		screen = grsegs[ERRORSCREEN];
	}
	
	ShutdownId();
	
	if (screen) {
		//DisplayTextSplash(screen);
	}
	
	if (error && *error) {
		fprintf(stderr, "Quit: %s\n", error);
    
    gfx_term();
	  display_delete(gameDisplay);
    control_term();

		exit(EXIT_FAILURE);
 	}

  gfx_term();
  display_delete(gameDisplay);
  control_term();

	exit(EXIT_SUCCESS);
}

void VL_WaitVBL(int vbls)
{
	unsigned long last = get_TimeCount() + vbls;
	while (last > get_TimeCount()) ;
}

void bitmap_scale(unsigned char *bmpline) {
  uint16_t* buffer_scr = gfx_render_target->address;
  unsigned int W,H,ix,iy,x,y;
  x=0;
  y=0;
  W=320;
  H=240;
  ix=(320<<16)/W;
  iy=(200<<16)/H;

  do   
  {
    unsigned char *buffer_mem=&bmpline[(y>>16)*320];
    W=320; x=0;
    do {
      *buffer_scr++=a320_pal[buffer_mem[x>>16]];
      x+=ix;
    } while (--W);
    y+=iy;
  } while (--H);
}

void VW_UpdateScreen()
{
  bitmap_scale(gfxbuf);
  display_flip(gameDisplay);
}

/*
=======================
=
= VL_Startup
=
=======================
*/

void VL_Startup()
{
	vwidth = 320;
	vheight = 200;
}

/*
=======================
=
= VL_Shutdown
=
=======================
*/

void VL_Shutdown()
{


	//SDL_Quit();
}

/* ======================================================================== */

/*
=================
=
= VL_SetPalette
=
=================
*/
typedef struct {
    int r;
    int g;
    int b;
} plt;

plt MyPalette[256];

void VL_SetPalette(const byte *palette)
{
	int i;
	
	VL_WaitVBL(1);
	
	for (i = 0; i < 256; i++)
	{
    a320_pal[i] = gfx_color_rgb( ((int)palette[i*3+0]*255)/63,((int)palette[i*3+1]*255)/63,((int)palette[i*3+2]*255)/63);
    MyPalette[i].r = palette[i*3+0];
    MyPalette[i].g = palette[i*3+1];
    MyPalette[i].b = palette[i*3+2];
	}
}

/*
=================
=
= VL_GetPalette
=
=================
*/

void VL_GetPalette(byte *palette)
{
	int i;
	for (i=0;i<256;i++)
	{
		palette[i*3+0] = MyPalette[i].r;
		palette[i*3+1] = MyPalette[i].g;
		palette[i*3+2] = MyPalette[i].b;
	}
}

static int XKeysymToScancode(unsigned int keysym)
{
  /*
	switch (keysym) {
		case SDLK_KP_ENTER:
		case SDLK_RETURN:
			return sc_Enter;
		case SDLK_ESCAPE:
			return sc_Escape;
		case SDLK_SPACE:
			return sc_Space;
		case SDLK_BACKSPACE:
			return sc_BackSpace;
		case SDLK_TAB:
			return sc_Tab;
		case SDLK_LALT:
		case SDLK_RALT:
			return sc_Alt;
		case SDLK_LCTRL:
		case SDLK_RCTRL:
			return sc_Control;
		case SDLK_CAPSLOCK:
			return sc_CapsLock;
		case SDLK_LSHIFT:
			return sc_LShift;
		case SDLK_RSHIFT:
			return sc_RShift;
		
		case SDLK_UP:
		case SDLK_KP8:
			return sc_UpArrow;
		case SDLK_DOWN:
		case SDLK_KP2:
			return sc_DownArrow;
		case SDLK_LEFT:
		case SDLK_KP4:
			return sc_LeftArrow;
		case SDLK_RIGHT:
		case SDLK_KP6:
			return sc_RightArrow;
		case SDLK_HOME:
			return sc_Home;
		case SDLK_END:
			return sc_End;
		case SDLK_PAGEUP:
			return sc_PgUp;
		case SDLK_PAGEDOWN:
			return sc_PgDn;
			
		case SDLK_INSERT:
			return sc_Insert;
		case SDLK_DELETE:
			return sc_Delete;
		case SDLK_F1:
			return sc_F1;
		case SDLK_F2:
			return sc_F2;
		case SDLK_F3:
			return sc_F3;
		case SDLK_F4:
			return sc_F4;
		case SDLK_F5:
			return sc_F5;
		case SDLK_F6:
			return sc_F6;
		case SDLK_F7:
			return sc_F7;
		case SDLK_F8:
			return sc_F8;
		case SDLK_F9:
			return sc_F9;
		case SDLK_F10:
			return sc_F10;
		case SDLK_F11:
			return sc_F11;
		case SDLK_F12:
			return sc_F12;	
		
		case SDLK_1:
			return sc_1;
		case SDLK_2:
			return sc_2;
		case SDLK_3:
			return sc_3;
		case SDLK_4:
			return sc_4;
		case SDLK_5:
			return sc_5;
		case SDLK_6:
			return sc_6;
		case SDLK_7:
			return sc_7;
		case SDLK_8:
			return sc_8;
		case SDLK_9:
			return sc_9;
		case SDLK_0:
			return sc_0;
		
		case SDLK_a:
			return sc_A;
		case SDLK_b:
			return sc_B;
		case SDLK_c:
			return sc_C;
		case SDLK_d:
			return sc_D;
		case SDLK_e:
			return sc_E;
		case SDLK_f:
			return sc_F;
		case SDLK_g:
			return sc_G;
		case SDLK_h:
			return sc_H;
		case SDLK_i:
			return sc_I;
		case SDLK_j:
			return sc_J;
		case SDLK_k:
			return sc_K;
		case SDLK_l:
			return sc_L;
		case SDLK_m:
			return sc_M;
		case SDLK_n:
			return sc_N;
		case SDLK_o:
			return sc_O;
		case SDLK_p:
			return sc_P;
		case SDLK_q:
			return sc_Q;
		case SDLK_r:
			return sc_R;
		case SDLK_s:
			return sc_S;
		case SDLK_t:
			return sc_T;
		case SDLK_u:
			return sc_U;
		case SDLK_v:
			return sc_V;
		case SDLK_w:
			return sc_W;
		case SDLK_x:
			return sc_X;
		case SDLK_y:
			return sc_Y;
		case SDLK_z:
			return sc_Z;
		
		case SDLK_PAUSE:
			return 0xE1;
		default:
			return sc_None;
	}
  */
}

extern int Volume;
void INL_Update()
{
  static byte volUp=0,volDown=0;
  
  control_poll();

	if ( control_check(CONTROL_BUTTON_X).pressed && control_check(CONTROL_DPAD_UP).pressed) {
    if (!volUp) {
      volUp=1;
      if (Volume < 100) Volume = Volume+10;
    }
  }
  else 
    volUp = 0;
	if ( control_check(CONTROL_BUTTON_X).pressed && control_check(CONTROL_DPAD_DOWN).pressed) {
    if (!volDown) {
      volDown=1;
      if (Volume > 0)  Volume = Volume-10;
    }
  }
  else 
    volDown = 0;
}

void IN_GetMouseDelta(int *dx, int *dy)
{
}

byte IN_MouseButtons()
{
	byte retr;
	
	retr = 0;

	return retr;
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_StartJoy() - Detects & auto-configures the specified joystick
//					The auto-config assumes the joystick is centered
//
///////////////////////////////////////////////////////////////////////////
boolean INL_StartJoy(word joy)
{
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_ShutJoy() - Cleans up the joystick stuff
//
///////////////////////////////////////////////////////////////////////////
void INL_ShutJoy(word joy)
{
}

/*
===================
=
= IN_JoyButtons
=
===================
*/

byte IN_JoyButtons()
{
	byte Buttons = 0;

	if ( control_check(CONTROL_BUTTON_A).pressed ) Buttons |= ( 1 << 0 );
	if ( control_check(CONTROL_BUTTON_B).pressed ) Buttons |= ( 1 << 1 );
	if ( control_check(CONTROL_BUTTON_X).pressed ) Buttons |= ( 1 << 2 );
	if ( control_check(CONTROL_BUTTON_Y).pressed ) Buttons |= ( 1 << 3 );
	if ( control_check(CONTROL_TRIGGER_LEFT).pressed ) Buttons |= ( 1 << 4 );
	if ( control_check(CONTROL_TRIGGER_RIGHT).pressed ) Buttons |= ( 1 << 5 );
	if ( control_check(CONTROL_BUTTON_START).pressed ) Buttons |= ( 1 << 6 );
	if ( control_check(CONTROL_BUTTON_SELECT).pressed ) Buttons |= ( 1 << 7 );

	return Buttons;
}

///////////////////////////////////////////////////////////////////////////
//
//	IN_GetJoyAbs() - Reads the absolute position of the specified joystick
//
///////////////////////////////////////////////////////////////////////////
void IN_GetJoyAbs(word joy,word *xp,word *yp)
{
	*xp = 0;
	*yp = 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_GetJoyDelta() - Returns the relative movement of the specified
//		joystick (from +/-127)
//
///////////////////////////////////////////////////////////////////////////
void INL_GetJoyDelta(word joy,int *dx,int *dy)
{
	if ( control_check(CONTROL_DPAD_LEFT).pressed ) *dx = -127;
	else if ( control_check(CONTROL_DPAD_RIGHT).pressed ) *dx = 127;
	else *dx = 0;

	if ( control_check(CONTROL_DPAD_UP).pressed ) *dy = -127;
	else if ( control_check(CONTROL_DPAD_DOWN).pressed ) *dy = 127;
	else *dy = 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_GetJoyButtons() - Returns the button status of the specified
//		joystick
//
///////////////////////////////////////////////////////////////////////////
word INL_GetJoyButtons(word joy)
{
  return IN_JoyButtons();
}

//
//	INL_SetJoyScale() - Sets up scaling values for the specified joystick
//
static void INL_SetJoyScale(word joy)
{
}

///////////////////////////////////////////////////////////////////////////
//
//      IN_SetupJoy() - Sets up thresholding values and calls INL_SetJoyScale()
//              to set up scaling values
//
///////////////////////////////////////////////////////////////////////////
void IN_SetupJoy(word joy,word minx,word maxx,word miny,word maxy)
{

}

extern int errno;

//int *__errno_location(void) __attribute__((weak));
int *__errno_location() { return &errno; }

void __assert_fail (const char *assertion, const char *file, unsigned int line, const char *function)
{
  abort();
}

int strcasecmp(const char *s1, const char *s2)
{
    while ((*s1 != '\0')
            && (tolower(*(unsigned char *) s1) ==
                tolower(*(unsigned char *) s2))) {
         s1++;
         s2++;
     }
 
     return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
}

char *strdup(const char *s) {
  char *tmp=(char *)malloc(strlen(s)+1);
  if (!tmp) return 0;
  strcpy(tmp,s);
  return tmp;
}

typedef long time_t;

time_t time(time_t *timeptr) {
  return OSTimeGet();
}

struct timeval 
{
  long tv_sec;                  // Seconds
  long tv_usec;                 // Microseconds
};

int gettimeofday(struct timeval *tv, void *tzp)
{
  long tt = OSTimeGet();
  tv->tv_sec = tt/100;
  tv->tv_usec = ((tt * 10000) % 1000000);
  return 0;
}


void IN_WaitForJoyMove( void ) {
	ControlInfo ci;

	do {
		IN_ReadControl( 0, &ci );
	}
	while ( ci.dir != dir_None );
}
